import os
import koji
import urllib
import subprocess

kojiurl = 'http://koji.fedoraproject.org/kojihub'
pkgurl = 'http://koji.fedoraproject.org/packages'

class RepoCreate(object):
    def __init__(self, repodir):
        self.repodir = repodir
        self.koji_connection = self.get_koji_connection()

    def get_koji_connection(self, url=kojiurl):
        return koji.ClientSession(url)


    def make_repo(self, packages, arches = ['x86_64', 'i386', 'i686', 'noarch']):
        self.prep_repo_dir()
        self.download_packages(packages, arches)
        self.make_repo_metadata()

    def prep_repo_dir(self):
        if not os.path.exists(self.repodir):
            os.makedirs(self.repodir)


    def make_repo_metadata(self):
        subprocess.call(['createrepo', self.repodir])


    def download_packages(self, packages, arches = ['x86_64', 'i386', 'i686', 'noarch']):
        rpms = []
        for package in packages:
            rpms.extend(self.get_rpm_urls(package, arches))

        for rpm in rpms:
            filename = '/'.join([self.repodir, rpm['url'].split('/')[-1]])
            urllib.urlretrieve(rpm['url'], filename)


    def get_build(self, nvr):
        return self.koji_connection.getBuild(nvr)


    def get_rpms(self, build_id, arches):
        rpms = self.koji_connection.listRPMs(buildID=build_id)
        return filter(lambda r: not r['name'].endswith('-debuginfo') and r['arch'] in arches, rpms)


    def get_rpm_urls(self, nvr, arches = ['x86_64', 'i386', 'i686', 'noarch']):
        # get build info
        build = self.get_build(nvr)

        # list rpms per build
        rpms = self.get_rpms(build['id'], arches)

        # generate list of urls
        baseurl = '/'.join((pkgurl, build['package_name'], build['version'],
                            build['release']))

        for rpm in rpms:
            rpm['url'] = "%s/%s" % (baseurl, koji.pathinfo.rpm(rpm))

        return rpms


if __name__ == '__main__':
    repodir = '/home/tflink/code/image_builder/testrepo'
    packages = [
                'kernel-3.3.0-0.rc5.git3.1.fc17',
                'dracut-017-17.git20120229.2.fc17',
                'anaconda-17.11-1.fc17',
                'pykickstart-1.99.7-1.fc17'
               ]

    repo = RepoCreate(repodir)
    repo.make_repo(packages, arches=['x86_64', 'noarch'])
